@@ -2,6 +2,8 @@
  * iSCSI Session Management and Slow-path Control
  *
  * Copyright (C) 2004 Dmitry Yusupov, Alex Aizman
+ * Copyright (C) 2006 Mike Christie
+ * Copyright (C) 2006 Red Hat, Inc. All rights reserved.
  * maintained by open-iscsi@googlegroups.com
  *
  * This program is free software; you can redistribute it and/or modify
@@ -584,7 +586,7 @@ session_conn_cleanup(queue_task_t *qtask, mgmt_ipc_err_e err)
 		__session_destroy(session);
 }
 
-static mgmt_ipc_err_e 
+static mgmt_ipc_err_e
 __session_conn_shutdown(iscsi_conn_t *conn, queue_task_t *qtask,
 			mgmt_ipc_err_e err)
 {
@@ -610,7 +612,7 @@ __session_conn_shutdown(iscsi_conn_t *conn, queue_task_t *qtask,
 	return MGMT_IPC_OK;
 }
 
-static mgmt_ipc_err_e 
+static mgmt_ipc_err_e
 session_conn_shutdown(iscsi_conn_t *conn, queue_task_t *qtask,
 		      mgmt_ipc_err_e err)
 {
@@ -660,7 +662,7 @@ __send_nopout(iscsi_conn_t *conn)
 		ISCSI_DIGEST_NONE, NULL, ISCSI_DIGEST_NONE, 0);
 }
 
-void 
+void
 __conn_noop_out_timeout(void *data)
 {
 	iscsi_conn_t *conn = (iscsi_conn_t*)data;
@@ -675,16 +677,16 @@ void
 __conn_noop_out(void *data)
 {
 	iscsi_conn_t *conn = (iscsi_conn_t*)data;
-	__send_nopout(conn);
+
 	if (conn->noop_out_timeout_timer.state == ACTOR_NOTSCHEDULED) {
+		__send_nopout(conn);
+
 		actor_timer(&conn->noop_out_timeout_timer,
 				conn->noop_out_timeout*1000,
 				__conn_noop_out_timeout, conn);
-		log_debug(3, "noop out timeout timer %p start\n",
-				&conn->noop_out_timeout_timer);
+		log_debug(3, "noop out timeout timer %p start, timeout %d\n",
+			 &conn->noop_out_timeout_timer, conn->noop_out_timeout);
 	}
-	actor_timer(&conn->noop_out_timer, conn->noop_out_interval*1000,
-			__conn_noop_out, data);
 }
 
 static void
@@ -1097,7 +1099,7 @@ static void iscsi_logout_timeout(void *data)
 	iscsi_conn_t *conn = data;
 
 	/*
-	 * assume we were in STATE_IN_LOGOUT or there 
+	 * assume we were in STATE_IN_LOGOUT or there
 	 * was some nasty error
 	 */
 	log_debug(3, "logout timeout, dropping conn...\n");
@@ -1131,10 +1133,13 @@ static int iscsi_send_logout(iscsi_conn_t *conn)
 
 static void iscsi_recv_nop_in(iscsi_conn_t *conn, struct iscsi_hdr *hdr)
 {
-	if (hdr->ttt == ISCSI_RESERVED_TAG)
+	if (hdr->ttt == ISCSI_RESERVED_TAG) {
 		/* noop out rsp */
 		actor_delete(&conn->noop_out_timeout_timer);
-	else /*  noop in req */
+		/* schedule a new ping */
+		actor_timer(&conn->noop_out_timer, conn->noop_out_interval*1000,
+			    __conn_noop_out, conn);
+	} else /*  noop in req */
 		if (!__send_nopin_rsp(conn, (struct iscsi_nopin*)hdr,
 				      conn->data)) {
 			log_error("can not send nopin response");
@@ -1213,7 +1218,7 @@ __session_conn_recv_pdu(queue_item_t *item)
 		if (iscsi_login_rsp(session, c)) {
 			log_debug(1, "login_rsp ret (%d)", c->ret);
 			if (c->ret != LOGIN_REDIRECT ||
-			    iscsi_login_redirect(conn))				
+			    iscsi_login_redirect(conn))
 				__session_conn_shutdown(conn, c->qtask,
 					MGMT_IPC_ERR_LOGIN_FAILURE);
 			return;
@@ -1256,14 +1261,17 @@ __session_conn_recv_pdu(queue_item_t *item)
 		}
 		break;
 	case STATE_XPT_WAIT:
+		recvpool_put(conn, conn->recv_handle);
 		log_debug(1, "ignoring incomming PDU in XPT_WAIT. "
 			  "let connection re-establish or fail");
 		break;
 	case STATE_CLEANUP_WAIT:
+		recvpool_put(conn, conn->recv_handle);
 		log_debug(1, "ignoring incomming PDU in XPT_WAIT. "
 			  "let connection cleanup");
 		break;
 	default:
+		recvpool_put(conn, conn->recv_handle);
 		log_error("Invalid state. Dropping PDU.\n");
 	}
 }
@@ -1364,12 +1372,12 @@ __session_conn_poll(queue_item_t *item)
 				   conn->id, conn->transport_ep_handle,
 				   (conn->id == 0), &rc) || rc) {
 			log_error("can't bind conn %d:%d to session %d, "
-				  "retcode %d (%d)", session->id, conn->id, 
+				  "retcode %d (%d)", session->id, conn->id,
 				   session->id, rc, errno);
 			err = MGMT_IPC_ERR_INTERNAL;
 			goto c_cleanup;
 		}
-		log_debug(3, "bound iSCSI connection %d:%d to session %d", 
+		log_debug(3, "bound iSCSI connection %d:%d to session %d",
 			  session->id, conn->id, session->id);
 
 		setup_kernel_io_callouts(conn);
@@ -1525,7 +1533,7 @@ __conn_error_handle(iscsi_session_t *session, iscsi_conn_t *conn)
 			}
 
 			break;
-		} 
+		}
 
 		/* mark all connections as failed */
 		for (i=0; i<ISCSI_CONN_MAX; i++) {
@@ -1572,7 +1580,7 @@ __session_conn_error(queue_item_t *item)
 	iscsi_session_t *session = conn->session;
 
 	log_warning("detected iSCSI connection %d:%d error (%d) "
-		    "state (%d)", session->id, conn->id, error, 
+		    "state (%d)", session->id, conn->id, error,
 		    conn->state);
 	__conn_error_handle(session, conn);
 
@@ -1642,6 +1650,31 @@ static int match_session(void *data, char *targetname, int tpgt, char *address,
 	return 0;
 }
 
+iscsi_session_t*
+session_find_by_sid(int sid)
+{
+	iscsi_provider_t *p;
+	iscsi_session_t *session;
+	struct qelem *pitem, *sitem;
+
+	pitem = providers.q_forw;
+	while (pitem != &providers) {
+		p = (iscsi_provider_t *)pitem;
+
+		sitem = p->sessions.q_forw;
+		while (sitem != &p->sessions) {
+			session = (iscsi_session_t *)sitem;
+
+			if (session->id == sid)
+				return session;
+			sitem = sitem->q_forw;
+		}
+		pitem = pitem->q_forw;
+	}
+
+	return NULL;
+}
+
 iscsi_session_t*
 session_find_by_rec(node_rec_t *rec)
 {
