@@ -2,6 +2,8 @@
  * iSCSI Administration Utility
  *
  * Copyright (C) 2004 Dmitry Yusupov, Alex Aizman
+ * Copyright (C) 2006 Mike Christie
+ * Copyright (C) 2006 Red Hat, Inc. All rights reserved.
  * maintained by open-iscsi@googlegroups.com
  *
  * This program is free software; you can redistribute it and/or modify
@@ -73,19 +75,20 @@ static struct option const long_options[] =
 	{"name", required_argument, NULL, 'n'},
 	{"value", required_argument, NULL, 'v'},
 	{"sid", required_argument, NULL, 'r'},
+	{"rescan", no_argument, NULL, 'R'},
+	{"info", no_argument, NULL, 'i'},
 	{"login", no_argument, NULL, 'l'},
 	{"loginall", required_argument, NULL, 'L'},
 	{"logout", no_argument, NULL, 'u'},
 	{"logoutall", required_argument, NULL, 'U'},
 	{"stats", no_argument, NULL, 's'},
 	{"debug", required_argument, NULL, 'g'},
-	{"map", required_argument, NULL, 'M'},
 	{"show", no_argument, NULL, 'S'},
 	{"version", no_argument, NULL, 'V'},
 	{"help", no_argument, NULL, 'h'},
 	{NULL, 0, NULL, 0},
 };
-static char *short_options = "lVhm:M:p:T:U:L:d:r:n:v:o:sSt:u";
+static char *short_options = "iRlVhm:p:T:U:L:d:r:n:v:o:sSt:u";
 
 static void usage(int status)
 {
@@ -98,7 +101,7 @@ iscsiadm -m discovery [ -dhV ] [ -t type -p ip:port [ -l ] ] | [ -p ip:port ] \
 [ -o operation ] [ -n name ] [ -v value ]\n\
 iscsiadm -m node [ -dhV ] [ -L all,manual,automatic ] [ -U all,manual,automatic ] [ -S ] [ [ -T targetname -p ip:port | -M sysdir ] [ -l | -u ] ] \
 [ [ -o  operation  ] [ -n name ] [ -v value ] [ -p ip:port ] ]\n\
-iscsiadm -m session [ -dhV ] [ -r sessionid [ -u | -s ] ]\n");
+iscsiadm -m session [ -dhV ] [ -r sessionid [ -i | -R | -u | -s ] [ -o operation ] [ -n name ] [ -v value ] ]\n");
 	}
 	exit(status == 0 ? 0 : -1);
 }
@@ -157,82 +160,6 @@ str_to_type(char *str)
 	return type;
 }
 
-static int
-sys_to_rec(idbm_t *db, node_rec_t *rec, char *sysfs_device)
-{
-	char *targetname;
-	char *address;
-	int sid, rc, port, tpgt, len;
-	struct stat statb;
-	char sys_session[64], *start, *last;
-
-	/*
-	 * Given sysfs_device is a directory name of the form:
-	 *
-	 * /sys/devices/platform/hostH/sessionS/targetH:B:I/H:B:I:L
-	 * /sys/devices/platform/hostH/sessionS/targetH:B:I
-	 * /sys/devices/platform/hostH/sessionS
-	 *
-	 * We want to set sys_session to sessionS
-	 */
-	if (stat(sysfs_device, &statb)) {
-		log_error("stat %s failed with %d", sysfs_device, errno);
-		exit(1);
-	}
-	if (!S_ISDIR(statb.st_mode)) {
-		log_error("%s is not a directory", sysfs_device);
-		exit(1);
-	}
-
-	last = NULL;
-	start = strstr(sysfs_device, "session");
-	if (start && strncmp(start, "session", 7) == 0) {
-		len = strlen(start);
-		last = index(start, '/');
-		/*
-		 * If '/' not found last is NULL.
-		 */
-		if (last)
-			len = last - start;
-		strncpy(sys_session, start, len);
-	} else {
-		log_error("Unable to find session in %s", sysfs_device);
-		exit(1);
-	}
-
-	targetname = malloc(TARGET_NAME_MAXLEN + 1);
-	if (!targetname)
-		return -ENOMEM;
-
-	address = malloc(NI_MAXHOST + 1);
-	if (!address) {
-		rc = -ENOMEM;
-		goto free_target;
-	}
-
-	log_debug(2, "%s: session %s", __FUNCTION__, sys_session);
-	rc = get_sessioninfo_by_sysfs_id(&sid, targetname, address, &port,
-					&tpgt, sys_session);
-	if (rc < 0) {
-		log_error("Unable to find a record for iscsi %s (sys %s)",
-			  sys_session, sysfs_device);
-		goto free_address;
-	}
-
-	rc = idbm_node_read(db, rec, targetname, address, port);
-	if (rc) {
-		log_error("node [%s, %s, %d] not found!",
-			  targetname, address, port);
-		goto free_address;
-	}
-
-free_address:
-	free(address);
-free_target:
-	free(targetname);
-	return rc;
-}
-
 static int
 session_login(node_rec_t *rec)
 {
@@ -332,9 +259,6 @@ __group_logout(void *data, char *targetname, int tpgt, char *address,
 	node_rec_t rec;
 	int rc = 0;
 
-	log_debug(7, "logout all [%d][%s,%s.%d]\n", sid, targetname,
-		  address, port);
-
 	/* for now skip qlogic and other HW and offload drivers */
 	if (!get_transport_by_sid(sid))
 		return 0;
@@ -356,10 +280,15 @@ __group_logout(void *data, char *targetname, int tpgt, char *address,
 		return 0;
 
 	if (!match_startup_mode(&rec, mode)) {
+		printf("Logout session [%d][%s:%d %s]\n", sid, address, port,
+			targetname);
+
 		rc = session_logout(&rec);
 		/* we raced with another app or instance of iscsiadm */
 		if (rc == MGMT_IPC_ERR_NOT_FOUND)
 			rc = 0;
+		if (rc)
+			log_error("Could not logout session (err %d).\n", rc);
 		if (rc > 0) {
 			iscsid_handle_error(rc);
 			/* continue trying to logout the rest of them */
@@ -396,8 +325,6 @@ __group_login(void *data, node_rec_t *rec)
 	char *mode = mgmt->mode;
 	int rc;
 
-	log_debug(7, "group login %s:%d,%d %s\n", rec->conn[0].address,
-		  rec->conn[0].port, rec->tpgt, rec->name);
 	/*
 	 * we always skip onboot because this should be handled by
 	 * something else
@@ -406,10 +333,15 @@ __group_login(void *data, node_rec_t *rec)
 		return 0;
 
 	if (!match_startup_mode(rec, mode)) {
+		printf("Login session [%s:%d %s]\n", rec->conn[0].address,
+			rec->conn[0].port, rec->name);
+
 		rc = session_login(rec);
 		/* we raced with another app or instance of iscsiadm */
 		if (rc == MGMT_IPC_ERR_EXISTS)
 			rc = 0;
+		if (rc)
+			log_error("Could not login session (err %d).\n", rc);
 		if (rc > 0) {
 			iscsid_handle_error(rc);
 			/* continue trying to login the rest of them */
@@ -490,28 +422,210 @@ config_init(void)
 	return 0;
 }
 
+static void print_scsi_device_info(int host_no, int target, int lun)
+{
+	char *blockdev, state[SCSI_MAX_STATE_VALUE];
+
+	printf("scsi%d Channel 00 Id %d Lun: %d\n", host_no, target, lun);
+	blockdev = get_blockdev_from_lun(host_no, target, lun);
+	if (blockdev) {
+		printf("Attached scsi disk %s\t\t", blockdev);
+		free(blockdev);
+
+		if (!get_device_state(state, host_no, target, lun))
+			printf("State: %s\n", state);
+		else
+			printf("State: Unknown\n");
+	}
+}
+
+static int is_valid_operational_value(int value)
+{
+	return value != -1;
+}
+
 static int print_session(void *data, char *targetname, int tpgt, char *address,
 			 int port, int sid)
 {
+	int extended = *((int *)data), host_no = -1, err = 0;
 	iscsi_provider_t *provider;
+	iscsiadm_req_t req;
+	iscsiadm_rsp_t rsp;
+	struct iscsi_session_operational_config session_conf;
+	struct iscsi_conn_operational_config conn_conf;
+	char state[SCSI_MAX_STATE_VALUE];
+        static char *conn_state[] = {
+			"FREE",
+			"TRANSPORT WAIT",
+			"IN LOGIN",
+			"LOGGED IN",
+			"IN LOGOUT",
+			"LOGOUT REQUESTED",
+			"CLEANUP WAIT",
+        };
+	static char *session_state[] = {
+			"NO CHANGE",
+			"CLEANUP",
+			"REPOEN",
+			"REDIRECT",
+	};
 
 	provider = get_transport_by_sid(sid);
 
-	printf("%s: [%d] %s:%d,%d %s\n", provider ? provider->name : "NA",
-		sid, address, port, tpgt, targetname);
+	if (!extended) {
+		printf("%s: [%d] %s:%d,%d %s\n",
+			provider ? provider->name : "NA",
+			sid, address, port, tpgt, targetname);
+		return 0;
+	}
+
+	/* TODO: how to pipe modinfo version info here */
+	printf("************************************\n");
+	printf("Session (sid %d) using module %s:\n", sid,
+		provider ? provider->name : "NA");
+	printf("************************************\n");
+	printf("TargetName: %s\n", targetname);
+	printf("Portal Group Tag: %d\n", tpgt);
+	printf("Network Portal: %s:%d\n", address, port);
+
+	get_negotiated_session_conf(sid, &session_conf);
+	get_negotiated_conn_conf(sid, &conn_conf);
+
+	/*
+	 * get iscsid's conn and session state. This may be slightly different
+	 * the kernel's view.
+	 *
+	 * TODO: get kernel state and qla4xxx info
+	 */
+	memset(&req, 0, sizeof(iscsiadm_req_t));
+	req.command = MGMT_IPC_SESSION_INFO;
+	req.u.session.sid = sid;
+
+	if (!do_iscsid(&ipc_fd, &req, &rsp)) {
+		/*
+		 * for drivers like qla4xxx, iscsid does not display anything
+		 * here since it does not know about it.
+		 */
+		if (rsp.u.session_state.conn_state < 0 ||
+		    rsp.u.session_state.conn_state > STATE_CLEANUP_WAIT)
+			printf("Invalid iSCSI Connection State\n");
+		else
+			printf("iSCSI Connection State: %s\n",
+				conn_state[rsp.u.session_state.conn_state]);
+
+		if (rsp.u.session_state.session_state < 0 ||
+		    rsp.u.session_state.session_state > R_STAGE_SESSION_REDIRECT)
+			printf("Invalid iscsid Session State\n");
+		else
+			printf("Internal iscsid Session State: %s\n",
+			      session_state[rsp.u.session_state.session_state]);
+	}
+
+	printf("\n");
+	printf("************************\n");
+	printf("Negotiated iSCSI params:\n");
+	printf("************************\n");
+
+	if (is_valid_operational_value(conn_conf.HeaderDigest))
+		printf("HeaderDigest: %s\n",
+			conn_conf.HeaderDigest ? "CRC32C" : "None");
+	if (is_valid_operational_value(conn_conf.DataDigest))
+		printf("DataDigest: %s\n",
+			conn_conf.DataDigest ? "CRC32C" : "None");
+	if (is_valid_operational_value(conn_conf.MaxRecvDataSegmentLength))
+		printf("MaxRecvDataSegmentLength: %d\n",
+			conn_conf.MaxRecvDataSegmentLength);
+	if (is_valid_operational_value(conn_conf.MaxXmitDataSegmentLength))
+		printf("MaxXmitDataSegmentLength: %d\n",
+			conn_conf.MaxXmitDataSegmentLength);
+	if (is_valid_operational_value(session_conf.FirstBurstLength))
+		printf("FirstBurstLength: %d\n",
+			session_conf.FirstBurstLength);
+	if (is_valid_operational_value(session_conf.MaxBurstLength))
+		printf("MaxBurstLength: %d\n",
+			session_conf.MaxBurstLength);
+	if (is_valid_operational_value(session_conf.ImmediateData))
+		printf("ImmediateData: %s\n",
+			session_conf.ImmediateData ? "Yes" : "No");
+	if (is_valid_operational_value(session_conf.InitialR2T))
+		printf("InitialR2T: %s\n",
+			session_conf.InitialR2T ? "Yes" : "No");
+	if (is_valid_operational_value(session_conf.MaxOutstandingR2T))
+		printf("MaxOutstandingR2T: %d\n",
+			session_conf.MaxOutstandingR2T);
+	printf("\n");
+
+	printf("************************\n");
+	printf("Attached SCSI devices:\n");
+	printf("************************\n");
+
+	host_no = get_host_no_from_sid(sid, &err);
+	if (err) {
+		printf("Host No: Unknown\n");
+		return err;
+	}
+	printf("Host Number: %d\t", host_no);
+	if (!get_host_state(state, host_no))
+		printf("State: %s\n", state);
+	else
+		printf("State: Unknown\n");
+	printf("\n");
+
+	sysfs_for_each_device(host_no, sid, print_scsi_device_info);
+	printf("\n");
+
 	return 0;
 }
 
-static int session_activelist(void)
+static int print_sessions(int extended)
+{
+	char version[20];
+	int num_found = 0, err = 0;
+
+	if (extended) {
+		if (get_iscsi_kernel_version(version))
+			printf("iSCSI Transport Class version %s\n",
+				version);
+		printf("%s version %s\n", program_name, ISCSI_VERSION_STR);
+	}
+
+	sysfs_for_each_session(&extended, &num_found, print_session);
+	if (err) {
+		log_error("Can not get list of active sessions (%d)", err);
+		return err;
+	} else if (!num_found)
+		log_error("no active sessions\n");
+	return 0;
+}
+
+static int rescan_session(void *data, char *targetname, int tpgt, char *address,
+			  int port, int sid)
+{
+	int host_no, err;
+
+	host_no = get_host_no_from_sid(sid, &err);
+	if (err) {
+		log_error("Could not rescan session sid %d\n", sid);
+		return err;
+	}
+
+	__scan_host(host_no, 0);
+	return 0;
+}
+
+static int rescan_sessions(void)
 {
 	int num_found = 0;
 
-	sysfs_for_each_session(NULL, &num_found, print_session);
-	return num_found;
+	sysfs_for_each_session(NULL, &num_found, rescan_session);
+	if (num_found <= 0)
+		return -ENODEV;
+	else
+		return 0;
 }
 
 static int
-session_stats(idbm_t *db, int sid)
+session_stats(int sid)
 {
 	int rc, i;
 	iscsiadm_req_t req;
@@ -640,12 +754,132 @@ static void catch_sigint( int signo ) {
 	exit(1);
 }
 
+static int exec_node_op(idbm_t *db, int op, int do_login, int do_logout,
+			int do_show, char *targetname,
+			char *ip, int port, char *name, char *value)
+{
+	int rc = 0;
+	node_rec_t rec;
+
+	memset(&rec, 0, sizeof(node_rec_t));
+
+	if (targetname && ip && op != OP_NEW) {
+		log_debug(2, "%s: node [%s,%s,%d]", __FUNCTION__,
+			  targetname, ip, port);
+		if (idbm_node_read(db, &rec, targetname, ip, port)) {
+			log_error("node [%s, %s, %d] not found!",
+				  targetname, ip, port);
+			rc = -1;
+			goto out;
+		}
+
+		if (do_login && do_logout) {
+			log_error("either login or "
+				  "logout at the time allowed!");
+			rc = -1;
+			goto out;
+		}
+
+		if ((do_login || do_logout) && op >= 0) {
+			log_error("either operation or login/logout "
+				  "at the time allowed!");
+			rc = -1;
+			goto out;
+		}
+
+		if (!do_login && !do_logout && op < 0) {
+			if (!idbm_print_node(db, &rec, do_show)) {
+				log_error("no records found!");
+				rc = -1;
+			}
+			goto out;
+
+		}
+
+		if (do_login) {
+			if ((rc = session_login(&rec)) > 0) {
+				iscsid_handle_error(rc);
+				rc = -1;
+			}
+			goto out;
+		}
+
+		if (do_logout) {
+			if ((rc = session_logout(&rec)) > 0) {
+				iscsid_handle_error(rc);
+				rc = -1;
+			}
+			goto out;
+		}
+
+		if (op == OP_UPDATE) {
+			if (!name || !value) {
+				log_error("update require name and "
+					  "value");
+				rc = -1;
+				goto out;
+			}
+
+			if ((rc = idbm_node_set_param(db, &rec,
+				      name, value))) {
+				log_error("can not set parameter");
+				goto out;
+			}
+		} else if (op == OP_DELETE) {
+			if (idbm_delete_node(db, &rec)) {
+				log_error("can not delete record");
+				rc = -1;
+				goto out;
+			}
+		} else {
+			log_error("operation is not supported.");
+			rc = -1;
+			goto out;
+		}
+	} else if (op < 0 || op == OP_SHOW) {
+		if (!idbm_for_each_node(db, NULL, print_node_info)) {
+			log_error("no records found!");
+			rc = -1;
+			goto out;
+		}
+	} else if (op == OP_NEW) {
+		if (!ip || !targetname) {
+			log_error("portal and target required for new "
+				  "node record");
+			rc = -1;
+			goto out;
+		}
+
+		idbm_node_setup_defaults(&rec);
+		strncpy(rec.name, targetname, TARGET_NAME_MAXLEN);
+		rec.conn[0].port = port;
+		strncpy(rec.conn[0].address, ip, NI_MAXHOST);
+		if (idbm_new_node(db, &rec)) {
+			log_error("can not add new record.");
+			rc = -1;
+			goto out;
+		}
+		printf("new iSCSI node record added\n");
+	} else if (op == OP_DELETE) {
+		log_error("--record required for delete operation");
+		rc = -1;
+		goto out;
+	} else {
+		log_error("operation is not supported.");
+		rc = -1;
+		goto out;
+	}
+
+out:
+	return rc;
+}
+
 int
 main(int argc, char **argv)
 {
-	char *ip = NULL, *name = NULL, *value = NULL, *sysfs_device = NULL;
+	char *ip = NULL, *name = NULL, *value = NULL;
 	char *targetname = NULL, *group_session_mgmt_mode = NULL;
-	int ch, longindex, mode=-1, port=-1, do_login=0;
+	int ch, longindex, mode=-1, port=-1, do_login=0, do_rescan=0, do_info=0;
 	int rc=0, sid=-1, op=-1, type=-1, do_logout=0, do_stats=0, do_show=0;
 	int do_login_all=0, do_logout_all=0;
 	idbm_t *db;
@@ -699,6 +933,12 @@ main(int argc, char **argv)
 				return -1;
 			}
 			break;
+		case 'R':
+			do_rescan = 1;
+			break;
+		case 'i':
+			do_info = 1;
+			break;
 		case 'l':
 			do_login = 1;
 			break;
@@ -725,9 +965,6 @@ main(int argc, char **argv)
 		case 'm':
 			mode = str_to_mode(optarg);
 			break;
-		case 'M':
-			sysfs_device = optarg;
-			break;
 		case 'T':
 			targetname = optarg;
 			break;
@@ -748,7 +985,7 @@ main(int argc, char **argv)
 		return -1;
 	}
 
-	if (mode < 0) 
+	if (mode < 0)
 		usage(0);
 
 	db = idbm_init(config_file);
@@ -857,10 +1094,8 @@ main(int argc, char **argv)
 			}
 		}
 	} else if (mode == MODE_NODE) {
-		node_rec_t rec;
-
-		memset(&rec, 0, sizeof(node_rec_t));
-		if ((rc = verify_mode_params(argc, argv, "dmMlSonvupTUL", 0))) {
+		if ((rc = verify_mode_params(argc, argv, "dmlSonvupTUL",
+					     0))) {
 			log_error("node mode: option '-%c' is not "
 				  "allowed/supported", rc);
 			rc = -1;
@@ -877,167 +1112,105 @@ main(int argc, char **argv)
 			goto out;
 		}
 
-		if (sysfs_device) {
-			if (targetname && ip) {
-				log_error("only one of map and "
-					  "targetname,portal may be specified");
-				rc = -1;
-				goto out;
-			}
-			if (sys_to_rec(db, &rec, sysfs_device)) {
-				rc = -1;
-				goto out;
-			}
-			/* bleh */
-			goto found_node_rec;
-		}
-
-		if (targetname && ip && op != OP_NEW) {
-			log_debug(2, "%s: node [%s,%s,%d]", __FUNCTION__,
-				  targetname, ip, port);
-			if (idbm_node_read(db, &rec, targetname, ip, port)) {
-				log_error("node [%s, %s, %d] not found!",
-					  targetname, ip, port);
-				rc = -1;
-				goto out;
-			}
-
-found_node_rec:
-			if (do_login && do_logout) {
-				log_error("either login or "
-					  "logout at the time allowed!");
-				rc = -1;
-				goto out;
-			}
-			if ((do_login || do_logout) && op >= 0) {
-				log_error("either operation or login/logout "
-					  "at the time allowed!");
-				rc = -1;
-				goto out;
-			}
-			if (!do_login && !do_logout && op < 0) {
-				if (!idbm_print_node(db, &rec, do_show)) {
-					log_error("no records found!");
-					rc = -1;
-				}
-				goto out;
-			}
-			if (do_login) {
-				if ((rc = session_login(&rec)) > 0) {
-					iscsid_handle_error(rc);
-					rc = -1;
-				}
-				goto out;
-			}
-			if (do_logout) {
-				if ((rc = session_logout(&rec)) > 0) {
-					iscsid_handle_error(rc);
-					rc = -1;
-				}
-				goto out;
-			}
-			if (op == OP_UPDATE) {
-				if (!name || !value) {
-					log_error("update require name and "
-						  "value");
-					rc = -1;
-					goto out;
-				}
-				if ((rc = idbm_node_set_param(db, &rec,
-					      name, value))) {
-					log_error("can not set parameter");
-					goto out;
-				}
-			} else if (op == OP_DELETE) {
-				if (idbm_delete_node(db, &rec)) {
-					log_error("can not delete record");
-					rc = -1;
-					goto out;
-				}
-			} else {
-				log_error("operation is not supported.");
-				rc = -1;
-				goto out;
-			}
-		} else if (op < 0 || op == OP_SHOW) {
-			if (!idbm_for_each_node(db, NULL, print_node_info)) {
-				log_error("no records found!");
-				rc = -1;
-				goto out;
-			}
-		} else if (op == OP_NEW) {
-			if (!ip || !targetname) {
-				log_error("portal and target required for new "
-					  "node record");
-				rc = -1;
-				goto out;
-			}
-			idbm_node_setup_defaults(&rec);
-			strncpy(rec.name, targetname, TARGET_NAME_MAXLEN);
-			rec.conn[0].port = port;
-			strncpy(rec.conn[0].address, ip, NI_MAXHOST);
-			if (idbm_new_node(db, &rec)) {
-				log_error("can not add new record.");
-				rc = -1;
-				goto out;
-			}
-			printf("new iSCSI node record added\n");
-		} else if (op == OP_DELETE) {
-			log_error("--record required for delete operation");
-			rc = -1;
-			goto out;
-		} else {
-			log_error("operation is not supported.");
-			rc = -1;
-			goto out;
-		}
+		rc = exec_node_op(db, op, do_login, do_logout, do_show,
+				  targetname, ip, port, name, value);
+		goto out;
 	} else if (mode == MODE_SESSION) {
-		if ((rc = verify_mode_params(argc, argv, "drmus", 1))) {
+		if ((rc = verify_mode_params(argc, argv, "iRdrmusonuSv", 1))) {
 			log_error("session mode: option '-%c' is not "
 				  "allowed or supported", rc);
 			rc = -1;
 			goto out;
 		}
 		if (sid >= 0) {
-			if (do_logout && do_stats) {
-				log_error("--logout or --stats? what exactly?");
-				rc = -1;
-				goto out;
-			}
-			if (do_logout) {
-				log_error("operation is not implemented yet.");
-				rc = -1;
+			char session[64];
+			int tmp_sid, tpgt;
+
+			if (do_rescan) {
+				rc = rescan_session(NULL, NULL, 0, NULL, 0,
+						    sid);
 				goto out;
 			}
+
 			if (do_stats) {
-				if ((rc = session_stats(db, sid)) > 0) {
+				if ((rc = session_stats(sid)) > 0) {
 					iscsid_handle_error(rc);
 					log_error("can not get statistics for "
 						"session with SID %d (%d)",
 						sid, rc);
 					rc = -1;
-					goto out;
 				}
+				goto out;
 			}
+
+			snprintf(session, 63, "session%d", sid);
+			session[63] = '\0';
+
+			targetname = malloc(TARGET_NAME_MAXLEN + 1);
+			if (!targetname) {
+				log_error("Could not allocate memory for "
+					  "targetname\n");
+				rc = -ENOMEM;
+				goto out;
+			}
+
+			ip = malloc(NI_MAXHOST + 1);
+			if (!ip) {
+				rc = -ENOMEM;
+				goto free_target;
+			}
+
+			rc = get_sessioninfo_by_sysfs_id(&tmp_sid, targetname,
+							ip, &port, &tpgt,
+							session);
+			if (rc) {
+				log_error("Could not get session info for sid "
+					  "%d\n", sid);
+				goto free_address;
+			}
+
+
+			if (do_info) {
+				int extended = 1;
+
+				rc = print_session(&extended, targetname,
+						   tpgt, ip, port, sid);
+				goto free_address;
+			}
+
+			/* drop down to node ops */
+			rc = exec_node_op(db, op, do_login, do_logout,
+					  do_show, targetname, ip,
+					  port, name, value);
+free_address:
+			free(ip);
+free_target:
+			free(targetname);
+			goto out;
 		} else {
 			if (do_logout) {
 				log_error("--logout requires target id");
 				rc = -1;
 				goto out;
 			}
+
 			if (do_stats) {
 				log_error("--stats requires target id");
 				rc = -1;
 				goto out;
 			}
-			if ((rc = session_activelist()) < 0) {
-				log_error("can not get list of active "
-					"sessions (%d)", rc);
-				rc = -1;
+
+			if (do_rescan) {
+				rc = rescan_sessions();
 				goto out;
-			} else if (!rc) {
-				printf("no active sessions\n");
 			}
+
+			if (do_info) {
+				rc = print_sessions(1);
+				goto out;
+			}
+
+			rc = print_sessions(0);
 		}
 	} else {
 		log_error("This mode is not yet supported");
